(function(global, undefined) {
    var filter = Array.prototype.filter;
    var forEach = Array.prototype.forEach;
    function getChildren(node) {
        return filter.call(node.childNodes || node.children, function(c) { return c.nodeType == 1 })
    }
    function getTextContent(node) {
        if (typeof node.textContent != 'undefined') {
            return node.textContent || "";
        } else {
            var res = "";
            forEach.call(node.childNodes || node.children, function(c) {
                if (c.nodeType == 3 || c.nodeType == 4) {
                    res += c.nodeValue || "";
                } else if (c.nodeType == 1) {
                    res += getTextContent(c);
                }
            });
            return res;
        }
    }

    var parsers = {

        "dict": function(node) {
            var children = getChildren(node)
              , res = {}
              , key = null;
            if (children.length % 2) {
                throw new Error("Malformed dict: dict must have even number of children");
            }
            children.forEach(function(n, i) {
                if (i % 2 == 0) {
                    if (n.nodeName != "key") {
                        throw new Error("Malformed dict: '" + n.nodeName + "' node found instead of 'key' node");
                    }
                    key = getTextContent(n);
                } else {
                    res[key] = parseSingle(n);
                }
            });
            return res;
        },

        "array": function(node) { return getChildren(node).map(parseSingle) },

        "string": function(node) { return getTextContent(node) },

        "integer": function(node) { return parseInt(getTextContent(node)) },

        "real": function(node) { return parseFloat(getTextContent(node)) },

        "true": function(node) { return true },

        "false": function(node) { return false },

        //"date": function(node) { return new Date(Date.parse(getTextContent(node))) },
        "date": function(node) { return getTextContent(node) }, // TODO: invent something

        "data": function(node) { return getTextContent(node) }, // TODO: support binary data somehow
    };

    function parseSingle(node) {
        var parse = parsers[node.nodeName];
        if (!parse) {
            throw new Error("Invalid node '" + node.nodeName + "'");
        }
        return parse(node);
    }

    global.parsePlist = function(doc) {
        var plists = getChildren(doc).filter(function(n) { return n.nodeName == "plist" });
        var roots = getChildren(plists[0]);

        if (roots.length != 1) {
            throw new Error("Plist document nust have exactly one child");
        }

        return parseSingle(roots[0]);
    } 
})(this);
