(function(global, document, $, ajax, JSON, App, tvKeyValue, widgetApi, pluginApi) {

    var S = function() {};

    S.prototype = new App.Scene();

    S.prototype.init = function(dom) {
        var self = this;

        // register this in global namespace
        self._globalName = "VideoPlayer" + (Math.random() * 1000000).toFixed();
        global[self._globalName] = self;

        self.plugin = document.getElementById("pluginPlayer");
        self.audioPlugin = document.getElementById("pluginAudio");
    }


    S.prototype.getUiLoading = function() {
        if (!this._uiLoading) {
            this._uiLoading = $('<DIV>')
                .addClass('loading')
                .css({'background': 'url("images/loading.png")'})
                .appendTo(this.dom);
        }
        return this._uiLoading;
    }

    S.prototype.showLoading = function() {
        var self = this
          , curBgOffset = 0
          , bgLength = 320
          , bgOffsetDelta = 40;
        
        if (self._rotateLoadingTimer) {
        	clearInterval(self._rotateLoadingTimer);
        	self._rotateLoadingTimer = null;
        }
        self._rotateLoadingTimer = setInterval(function() {
            curBgOffset = (curBgOffset + bgOffsetDelta) % bgLength;
            self.getUiLoading().css({
                backgroundPosition: -curBgOffset + "px 0px"
            });
        }, 100);
        self.getUiLoading().show();
    }

    S.prototype.hideLoading = function() {
        this.getUiLoading().hide();
        if (this._rotateLoadingTimer) {
            clearInterval(this._rotateLoadingTimer);
        }
    }

    S.prototype.startVideo = function(url) {
		alert("Starting: " + url);
        var self = this;
		self.plugin.OnStreamInfoReady = self._globalName+".onPlayerStreamInfoReady";

		self.plugin.OnBufferingComplete = self._globalName+".onPlayerBufferingComplete";
		self.plugin.OnBufferingStart = self._globalName+".onPlayerBufferingStart";
		self.plugin.OnBufferingProgress = self._globalName+".onPlayerBufferingProgress";
		self.plugin.OnRenderingComplete = self._globalName+".onPlayerRenderingComplete";
		
		self.plugin.OnRenderError = self._globalName+".onPlayerRenderError";
		self.plugin.OnStreamNotFound = self._globalName+".onPlayerStreamNotFound";
		self.plugin.OnNetworkDisconnected = self._globalName+".onPlayerNetworkDisconnected";

        pluginApi.setOffScreenSaver();

        if (self.onBeforePlay) {
            self.onBeforePlay();
        }

        self.plugin.Play(url);
        //self.plugin.Play("http://live.kp.cdnvideo.ru/kp-live/test/playlist.m3u8|COMPONENT=HLS");
        self.currentUrl = url;
    }

    S.prototype._startRetry = function() {
        var self = this;
        if (self._retryTimer != null) {
            clearInterval(self._retryTimer);
        }
        var retry = function() {
            self.stopVideo();
            self.startVideo(self.currentUrl);
            self._retryTimer = setTimeout(retry, 3000);
        };
        self._retryTimer = setTimeout(retry, 3000);
    }

    S.prototype._clearRetryTimer = function() {
        if (this._retryTimer != null) {
            clearInterval(this._retryTimer);
            this._retryTimer = null;
        }
    }

    S.prototype.setVideoRegion = function(l, t, w, h) {
        this._l = l;
        this._t = t;
        this._w = w;
        this._h = h;
    }

    S.prototype.onPlayerStreamInfoReady = function() {
        if (this.plugin.SetDisplayArea) {

            alert("set display area");

            var areaL = this._l || 0
              , areaT = this._t || 0
              , areaW = this._w || this.dom.width()
              , areaH = this._h || this.dom.height()
              , videoW = this.plugin.GetVideoWidth() || areaW
              , videoH = this.plugin.GetVideoHeight() || areaH
              , targetL = areaL
              , targetT = areaT
              , targetW = areaW
              , targetH = areaH;

            alert("Video size is: " + videoW + "x" + videoH);

            if (videoW * areaH < areaW * videoH) {
                // areaH is too small
                targetW = videoW * areaH / videoH;
            } else if (areaW * videoH < videoW * areaH) {
                // areaW is too small
                targetH = areaW * videoH / videoW;
            }

            targetL = Math.round(areaL + (areaW - targetW)/2);
            targetT = Math.round(areaT + (areaH - targetH)/2);
            targetW = Math.round(targetW);
            targetH = Math.round(targetH);

            alert(targetL + " " + targetT + " " + targetW + " " + targetH);

            // repeat twice (can't remember why it is necessary :)
            this.plugin.SetDisplayArea(targetL, targetT, targetW, targetH);
            this.plugin.SetDisplayArea(targetL, targetT, targetW, targetH);
        }
    }

    S.prototype.onPlayerBufferingStart = function() {
        this.showLoading();
		//this._retryCount = 0;
		this.hideMessage();
        alert('buffering');
    }
	
	S.prototype.onPlayerBufferingProgress = function() {
		this.hideMessage();
        this._clearRetryTimer();
	}
	
	S.prototype.onPlayerBufferingComplete = function() {
		this.hideMessage();
        this.hideLoading();
        this._clearRetryTimer();
        alert('buffered');
	}

	S.prototype.onPlayerRenderError = function() {
        var self = this;
        self.showMessage("Ошибка воспроизведения");
		self.stopVideo();
        self._startRetry();
        alert('error');
		/*if (this._retryCount < 3) {
			this._retryCount++;
			this.startVideo(this.currentUrl);
		}*/
	}

	S.prototype.onPlayerStreamNotFound = function() {
        var self = this;
        self.showMessage("Поток не найден");
		self.stopVideo();
        self._startRetry();
        alert('not found');
	}

	S.prototype.onPlayerNetworkDisconnected = function() {
        var self = this;
        self.showMessage("Сеть отключилась");
		self.stopVideo();
        self._startRetry();
        alert('disconnected');
	}

    S.prototype.stopVideo = function() {
        var self = this;
        self.plugin.Stop();

		self.plugin.OnBufferingComplete = null;
		self.plugin.OnBufferingStart = null;
		self.plugin.OnRenderingComplete = null;
		
		self.plugin.OnRenderError = null;
		self.plugin.OnStreamNotFound = null;
		self.plugin.OnNetworkDisconnected = null;

        pluginApi.setOnScreenSaver();
    }

    S.prototype.finish = function() {}
    S.prototype.onPause = function() {
        this._clearRetryTimer();
    }

    S.prototype.onKeyDown = function(keyCode) {
        // There was a volume handling code here
        // Now it is not needed
        return false;
    }

    global.VideoScene = S;

})(this, document, jQuery, ajax, JSON, App, App.tvKeyValue, App.widgetApi, App.pluginApi);
